﻿class Validation {
    constructor() {
        this.validationMsg = '#auto-save-details';
        this.errors = [];
    }

    validateForm($form) {
        let valid = true;
        var self = this;

        this.getFields($form).each(function () {
            if (!self.validateField($(this), true)) {
                valid = false;
            }
        });

        this.showErrors();

        return valid;
    }

    validateField(field, suppressErrorDisplay) {
        let displayFieldId = field.data('display-field-id');    // For validating hidden inputs with an alternate display field visible
        let displayField = displayFieldId ? $('#' + displayFieldId) : field;
        let valid = true;
        let fieldName = field.attr('name');
        var value = field.val();

        if (!fieldName || field.prop('readonly'))    // If the field isn't being passed to the server or is readonly, ignore it
            return valid;

        displayField.removeClass('invalid');

        if (field.attr('type') === 'checkbox')
            value = field.prop('checked');

        if (fieldName.split('.').length > 1)
            fieldName = fieldName.split('.')[1];    // Remove prefixes (Form., contact., etc.)

        if (field.prop('required') && (value === '' || !value)) {
            this.errors.push(fieldName + ' is required.');
            valid = false;
        }

        let dataType = field.data('field-type');
        if (dataType && valid && value) {    // Don't bother checking type if we failed the required check and/or it is empty
            switch (dataType) {
                case 'int':
                    if (!this.validateInt(value)) {
                        valid = false;
                        this.errors.push(fieldName + ' must be a valid integer.');
                    }
                    break;
                case 'decimal':
                case 'double':
                    if (!this.validateDecimal(value)) {
                        valid = false;
                        this.errors.push(fieldName + ' must be a valid number.');
                    }
                    break;
                case 'date':
                    if (!this.validateDate(value)) {
                        valid = false;
                        this.errors.push(fieldName + ' must be a valid date in the format M/D/YYYY.');
                    }
                    break;
                case 'email':
                    if (!this.validateEmail(value)) {
                        valid = false;
                        this.errors.push(fieldName + ' must be a valid email address.');
                    }
                    break;
                case 'ssn':
                    if (!this.validateSsn(value)) {
                        valid = false;
                        this.errors.push(fieldName + ' must be a valid SSN.');
                    }
                    break;
            }
        }

        if (!valid) {
            displayField.addClass('invalid');
        }

        if (!suppressErrorDisplay)
            this.showErrors();

        return valid;
    }

    validateInt(val) {
        return val.match(/^\$?[-+]?[0-9]+(?:,\d{3})*%?$/i);
    }

    validateDecimal(val) {
        return val.match(/^\$?[-+]?[0-9]+(?:,\d{3})*(?:\.[0-9]+)?%?$/i);
    }

    validateEmail(val) {
        return val.match(/^.+@.+\..+$/i);
    }

    validateSsn(val) {
        return val.match(/^(?:\d{3}-\d{2}-\d{4}|\d{9})$/i);
    }

    validateDate(val) {
        // First check for the pattern
        if (!/^\d{1,2}\/\d{1,2}\/\d{4}$/.test(val))
            return false;

        // Parse the date parts to integers
        var parts = val.split("/");
        var day = parseInt(parts[1], 10);
        var month = parseInt(parts[0], 10);
        var year = parseInt(parts[2], 10);

        // Check the ranges of month and year
        if (year < 1000 || year > 3000 || month < 1 || month > 12)
            return false;

        var monthLength = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

        // Adjust for leap years
        if (year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0))
            monthLength[1] = 29;

        // Check the range of the day
        return day > 0 && day <= monthLength[month - 1];
    }

    showErrors() {
        if (this.errors.length) {
            let msg = '';
            for (let i = 0; i < this.errors.length; i++) {
                msg += this.errors[i] + '<br />';
            }
            this.showMessage(msg);
        }
    }

    showMessage(msg) {
        $(this.validationMsg).html(msg);
    }

    sanitizeFieldValue(field) {
        if (field.attr('type') === 'checkbox')
            return field.prop('checked');

        let fieldType = field.data('field-type');
        let value = field.val();

        if (!fieldType)
            return value;

        if (fieldType === 'decimal' && value.startsWith('$'))
            return value.substr(1);    // Trim leading $

        if (fieldType === 'double' && value.endsWith('%'))
            return parseFloat(value) / 100.0;    // Raw text not from a dropdown

        return value;
    }

    sanitizeForm($form) {
        let sanitizeFieldValue = this.sanitizeFieldValue;
        this.getFields($form).each(function () {
            $(this).val(sanitizeFieldValue($(this)));
        });
    }

    getFields($form) {
        if ($form.length > 1)
            return $form;    // Pre-filtered inputs from the BulkEdit tool
        else
            return $('input, select, textarea', $form);    // A regular form submission
    }
}